@@ -21,7 +21,12 @@
 
 from __future__ import annotations
 
-from urwid import raw_display
+import importlib
+import sys
+import types
+import typing
+import warnings
+
 from urwid.canvas import (
     BlankCanvas,
     Canvas,
@@ -48,7 +53,7 @@ from urwid.command_map import (
     CommandMap,
     command_map,
 )
-from urwid.display_common import (
+from urwid.display import (
     BLACK,
     BROWN,
     DARK_BLUE,
@@ -90,7 +95,15 @@ from urwid.font import (
 )
 from urwid.listbox import ListBox, ListBoxError, ListWalker, ListWalkerError, SimpleFocusListWalker, SimpleListWalker
 from urwid.monitored_list import MonitoredFocusList, MonitoredList
-from urwid.signals import MetaSignals, Signals, connect_signal, disconnect_signal, emit_signal, register_signal
+from urwid.signals import (
+    MetaSignals,
+    Signals,
+    connect_signal,
+    disconnect_signal,
+    disconnect_signal_by_key,
+    emit_signal,
+    register_signal,
+)
 from urwid.text_layout import LayoutSegment, StandardTextLayout, TextLayout, default_layout
 from urwid.util import (
     MetaSuper,
@@ -111,8 +124,8 @@ from urwid.util import (
     supports_unicode,
     within_double_byte,
 )
-from urwid.version import __version__, __version_tuple__
-from urwid.vterm import TermCanvas, TermCharset, Terminal, TermModes, TermScroller
+from urwid.version import version as __version__
+from urwid.version import version_tuple as __version_tuple__
 from urwid.widget import (
     ANY,
     BOTTOM,
@@ -194,7 +207,7 @@ from urwid.widget import (
     scale_bar_values,
 )
 
-from . import event_loop, widget
+from . import display, event_loop, widget
 
 from urwid.treetools import ParentNode, TreeListBox, TreeNode, TreeWalker, TreeWidget, TreeWidgetError  # isort: skip
 
@@ -220,10 +233,104 @@ try:
 except ImportError:
     pass
 
-try:
-    from urwid.event_loop import ZMQEventLoop
-except ImportError:
-    pass
+# OS Specific
+if sys.platform != "win32":
+    from urwid.vterm import TermCanvas, TermCharset, Terminal, TermModes, TermScroller
+
+    # ZMQEventLoop cause interpreter crash on windows
+    try:
+        from urwid.event_loop import ZMQEventLoop
+    except ImportError:
+        pass
 
 # Backward compatibility
 VERSION = __version_tuple__
+
+
+# Moved modules handling
+__locals: dict[str, typing.Any] = locals()  # use mutable access for pure lazy loading
+
+# Backward compatible lazy load with deprecation warnings
+_moved_warn: dict[str, str] = {
+    "lcd_display": "urwid.display.lcd",
+    "html_fragment": "urwid.display.html_fragment",
+    "web_display": "urwid.display.web",
+}
+# Backward compatible lazy load without any warnings
+# Before DeprecationWarning need to start PendingDeprecationWarning process.
+_moved_no_warn: dict[str, str] = {
+    "display_common": "urwid.display.common",
+    "raw_display": "urwid.display.raw",
+    "curses_display": "urwid.display.curses",
+    "escape": "urwid.display.escape",
+}
+
+
+class _MovedModule(types.ModuleType):
+    """Special class to handle moved modules.
+
+    PEP-0562 handles moved modules attributes, but unfortunately not handle nested modules access
+    like "from xxx.yyy import zzz"
+    """
+
+    __slots__ = ("_moved_from", "_moved_to")
+
+    def __init__(self, moved_from: str, moved_to: str) -> None:
+        super().__init__(moved_from.join(".")[-1])
+        self._moved_from = moved_from
+        self._moved_to = moved_to
+
+    def __getattr__(self, name: str) -> typing.Any:
+        real_module = importlib.import_module(self._moved_to)
+        sys.modules[self._moved_from] = real_module
+        return getattr(real_module, name)
+
+
+class _MovedModuleWarn(_MovedModule):
+    """Special class to handle moved modules.
+
+    Produce DeprecationWarning messages for imports.
+    """
+
+    __slots__ = ()
+
+    def __getattr__(self, name: str) -> typing.Any:
+        warnings.warn(
+            f"{self._moved_from} is moved to {self._moved_to}",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return super().__getattr__(name)
+
+
+for _name, _module in _moved_no_warn.items():
+    _module_path = f"{__name__}.{_name}"
+    sys.modules[_module_path] = _MovedModule(_module_path, _module)
+
+for _name, _module in _moved_warn.items():
+    _module_path = f"{__name__}.{_name}"
+    sys.modules[_module_path] = _MovedModuleWarn(_module_path, _module)
+
+
+def __getattr__(name: str) -> typing.Any:
+    """Get attributes lazy.
+
+    :return: attribute by name
+    :raises AttributeError: attribute is not defined for lazy load
+    """
+    if name in _moved_no_warn:
+        mod = importlib.import_module(_moved_no_warn[name])
+        __locals[name] = mod
+        return mod
+
+    if name in _moved_warn:
+        warnings.warn(
+            f"{name} is moved to {_moved_warn[name]}",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
+        mod = importlib.import_module(_moved_warn[name])
+        __locals[name] = mod
+        return mod
+    raise AttributeError(f"{name} not found in {__package__}")
