@@ -21,6 +21,7 @@
 from __future__ import annotations
 
 import functools
+import logging
 import typing
 import warnings
 from operator import attrgetter
@@ -34,7 +35,10 @@ from urwid.util import MetaSuper
 from .constants import Sizing
 
 if typing.TYPE_CHECKING:
-    from collections.abc import Callable
+    from collections.abc import Callable, Hashable
+
+
+LOGGER = logging.getLogger(__name__)
 
 
 class WidgetMeta(MetaSuper, signals.MetaSignals):
@@ -73,7 +77,11 @@ class WidgetMeta(MetaSuper, signals.MetaSignals):
 
 
 class WidgetError(Exception):
-    pass
+    """Widget specific errors."""
+
+
+class WidgetWarning(Warning):
+    """Widget specific warnings."""
 
 
 def validate_size(widget, size, canv):
@@ -279,16 +287,10 @@ class Widget(metaclass=WidgetMeta):
 
     .. method:: keypress(size, key)
 
-       .. note::
-
-          This method is not implemented in :class:`.Widget` but
-          must be implemented by any selectable widget.
-          See :meth:`.selectable`.
-
        :param size: See :meth:`Widget.render` for details
        :type size: widget size
        :param key: a single keystroke value; see :ref:`keyboard-input`
-       :type key: bytes or unicode
+       :type key: str
 
        :returns: ``None`` if *key* was handled by this widget or
                  *key* (the same value passed) if *key* was not handled
@@ -310,13 +312,6 @@ class Widget(metaclass=WidgetMeta):
 
     .. method:: mouse_event(size, event, button, col, row, focus)
 
-       .. note::
-
-          This method is not implemented in :class:`.Widget` but
-          may be implemented by a subclass.  Not implementing this
-          method is equivalent to having a method that always returns
-          ``False``.
-
        :param size: See :meth:`Widget.render` for details.
        :type size: widget size
        :param event: Values such as ``'mouse press'``, ``'ctrl mouse press'``,
@@ -415,6 +410,9 @@ class Widget(metaclass=WidgetMeta):
     _sizing = frozenset([Sizing.FLOW, Sizing.BOX, Sizing.FIXED])
     _command_map = command_map
 
+    def __init__(self):
+        self.logger = logging.getLogger(f"{self.__class__.__module__}.{self.__class__.__name__}")
+
     def _invalidate(self) -> None:
         """
         Mark cached canvases rendered by this widget as dirty so that
@@ -422,7 +420,7 @@ class Widget(metaclass=WidgetMeta):
         """
         CanvasCache.invalidate(self)
 
-    def _emit(self, name: str, *args):
+    def _emit(self, name: Hashable, *args):
         """
         Convenience function to emit signals with self as first
         argument.
@@ -572,6 +570,69 @@ class Widget(metaclass=WidgetMeta):
     def _repr_attrs(self):
         return {}
 
+    def keypress(self, size: tuple[()] | tuple[int] | tuple[int, int], key: str) -> str | None:
+        """Keyboard input handler.
+
+        :param size: See :meth:`Widget.render` for details
+        :type size: tuple[()] | tuple[int] | tuple[int, int]
+        :param key: a single keystroke value; see :ref:`keyboard-input`
+        :type key: str
+        :return: ``None`` if *key* was handled by *key* (the same value passed) if *key* was not handled
+        :rtype: str | None
+        """
+        if not self.selectable():
+            if hasattr(self, "logger"):
+                self.logger.debug(f"keypress sent to non selectable widget {self!r}")
+            else:
+                warnings.warn(
+                    f"Widget {self.__class__.__name__} did not call 'super().__init__()",
+                    WidgetWarning,
+                    stacklevel=3,
+                )
+                LOGGER.debug(f"Widget {self!r} is not selectable")
+        return key
+
+    def mouse_event(
+        self,
+        size: tuple[()] | tuple[int] | tuple[int, int],
+        event: str,
+        button: int,
+        col: int,
+        row: int,
+        focus: bool,
+    ) -> bool | None:
+        """Mouse event handler.
+
+        :param size: See :meth:`Widget.render` for details.
+        :type size: tuple[()] | tuple[int] | tuple[int, int]
+        :param event: Values such as ``'mouse press'``, ``'ctrl mouse press'``,
+                     ``'mouse release'``, ``'meta mouse release'``,
+                     ``'mouse drag'``; see :ref:`mouse-input`
+        :type event: str
+        :param button: 1 through 5 for press events, often 0 for release events
+                      (which button was released is often not known)
+        :type button: int
+        :param col: Column of the event, 0 is the left edge of this widget
+        :type col: int
+        :param row: Row of the event, 0 it the top row of this widget
+        :type row: int
+        :param focus: Set to ``True`` if this widget or one of its children is in focus
+        :type focus: bool
+        :return: ``True`` if the event was handled by this widget, ``False`` otherwise
+        :rtype: bool | None
+        """
+        if not self.selectable():
+            if hasattr(self, "logger"):
+                self.logger.debug(f"Widget {self!r} is not selectable")
+            else:
+                warnings.warn(
+                    f"Widget {self.__class__.__name__} not called 'super().__init__()",
+                    WidgetWarning,
+                    stacklevel=3,
+                )
+                LOGGER.debug(f"Widget {self!r} is not selectable")
+        return False
+
 
 class FlowWidget(Widget):
     """
@@ -598,6 +659,7 @@ class FlowWidget(Widget):
             DeprecationWarning,
             stacklevel=3,
         )
+        super().__init__()
 
     def rows(self, size: int, focus: bool = False) -> int:
         """
@@ -640,6 +702,7 @@ class BoxWidget(Widget):
             DeprecationWarning,
             stacklevel=3,
         )
+        super().__init__()
 
     def render(self, size: tuple[int, int], focus: bool = False):
         """
@@ -683,6 +746,7 @@ class FixedWidget(Widget):
             DeprecationWarning,
             stacklevel=3,
         )
+        super().__init__()
 
     def render(self, size, focus=False):
         """
@@ -697,7 +761,7 @@ class FixedWidget(Widget):
         raise NotImplementedError()
 
 
-def delegate_to_widget_mixin(attribute_name: str):
+def delegate_to_widget_mixin(attribute_name: str) -> type[Widget]:
     """
     Return a mixin class that delegates all standard widget methods
     to an attribute given by attribute_name.
@@ -721,37 +785,40 @@ def delegate_to_widget_mixin(attribute_name: str):
             return get_delegate(self).selectable
 
         @property
-        def get_cursor_coords(self):
+        def get_cursor_coords(self) -> Callable[[tuple[()] | tuple[int] | tuple[int, int]], tuple[int, int] | None]:
+            # TODO(Aleksei):  Get rid of property usage after getting rid of "if getattr"
             return get_delegate(self).get_cursor_coords
 
         @property
-        def get_pref_col(self):
+        def get_pref_col(self) -> Callable[[tuple[()] | tuple[int] | tuple[int, int]], int | None]:
+            # TODO(Aleksei):  Get rid of property usage after getting rid of "if getattr"
             return get_delegate(self).get_pref_col
 
-        @property
-        def keypress(self):
-            return get_delegate(self).keypress
+        def keypress(self, size: tuple[()] | tuple[int] | tuple[int, int], key: str) -> str | None:
+            return get_delegate(self).keypress(size, key)
 
         @property
-        def move_cursor_to_coords(self):
+        def move_cursor_to_coords(self) -> Callable[[[tuple[()] | tuple[int] | tuple[int, int], int, int]], bool]:
+            # TODO(Aleksei):  Get rid of property usage after getting rid of "if getattr"
             return get_delegate(self).move_cursor_to_coords
 
         @property
-        def rows(self):
+        def rows(self) -> Callable[[tuple[int], bool], int]:
             return get_delegate(self).rows
 
         @property
         def mouse_event(
             self,
-        ) -> Callable[[tuple[()] | tuple[int] | tuple[int, int], str, int, int, int, bool], bool | None,]:
+        ) -> Callable[[tuple[()] | tuple[int] | tuple[int, int], str, int, int, int, bool], bool | None]:
+            # TODO(Aleksei):  Get rid of property usage after getting rid of "if getattr"
             return get_delegate(self).mouse_event
 
         @property
-        def sizing(self):
+        def sizing(self) -> Callable[[], frozenset[Sizing]]:
             return get_delegate(self).sizing
 
         @property
-        def pack(self):
+        def pack(self) -> Callable[[tuple[()] | tuple[int] | tuple[int, int], bool], tuple[int, int]]:
             return get_delegate(self).pack
 
     return DelegateToWidgetMixin
@@ -761,7 +828,7 @@ class WidgetWrapError(Exception):
     pass
 
 
-class WidgetWrap(delegate_to_widget_mixin("_wrapped_widget"), Widget):
+class WidgetWrap(delegate_to_widget_mixin("_wrapped_widget")):
     def __init__(self, w: Widget):
         """
         w -- widget to wrap, stored as self._w
@@ -776,6 +843,14 @@ class WidgetWrap(delegate_to_widget_mixin("_wrapped_widget"), Widget):
         of the wrapped widgets by behaving like a ContainerWidget or
         WidgetDecoration, or it may hide them from outside access.
         """
+        super().__init__()
+        if not isinstance(w, Widget):
+            obj_class_path = f"{w.__class__.__module__}.{w.__class__.__name__}"
+            warnings.warn(
+                f"{obj_class_path} is not subclass of Widget",
+                DeprecationWarning,
+                stacklevel=2,
+            )
         self._wrapped_widget = w
 
     @property
